@@ -89,6 +89,10 @@ struct _GPdfControlPrivate {
 	
 	BonoboControl *bonobo_page_control;
 
+	GtkWidget *file_chooser;
+	GtkWidget *properties;
+	GtkWidget *print_dialog;
+
 	guint idle_id;
 };
 
@@ -972,7 +976,7 @@ gpdf_control_print (GPdfControl *control)
 	GPdfPrintJob *pj;
 	GnomePrintConfig *config;
 	GnomePrintJob *job;
-	GtkWidget *print_dialog;
+	GPdfControlPrivate *priv;
 	gint response;
 	GdkWindow *view_window;
 	GdkDisplay *display;
@@ -984,12 +988,11 @@ gpdf_control_print (GPdfControl *control)
 	config = gnome_print_config_default ();
 	job = gnome_print_job_new (config);
 
-	print_dialog = gpdf_control_print_dialog_new (control, job);
-	g_signal_connect_swapped (G_OBJECT (control), "disconnected",
-				  G_CALLBACK (gtk_widget_destroy),
-				  GTK_WIDGET (print_dialog));
-	response = gtk_dialog_run (GTK_DIALOG (print_dialog));
-	gtk_widget_hide (print_dialog);
+	priv = control->priv;
+	
+	priv->print_dialog = gpdf_control_print_dialog_new (control, job);
+	response = gtk_dialog_run (GTK_DIALOG (priv->print_dialog));
+	gtk_widget_hide (priv->print_dialog);
 
 	if (response != GNOME_PRINT_DIALOG_RESPONSE_PRINT)
 		goto exit;
@@ -1002,10 +1005,11 @@ gpdf_control_print (GPdfControl *control)
 	pj = g_new0 (GPdfPrintJob, 1);
 
 	gpdf_control_get_range_page (
-		control, GNOME_PRINT_DIALOG (print_dialog),
+		control, GNOME_PRINT_DIALOG (priv->print_dialog),
 		&pj->page_start, &pj->page_end);
 
-	gtk_widget_destroy (print_dialog);
+	gtk_widget_destroy (priv->print_dialog);
+	priv->print_dialog = NULL;
 
 	print_to_file_workaround (job);
 
@@ -1030,41 +1034,47 @@ exit:
 static gchar *
 gpdf_control_get_filename (BonoboControl *control)
 {
-	GtkWidget *filechooser;
+	GPdfControlPrivate *priv;
 	GtkFileFilter *pdf_filter, *all_filter;
 	gchar *pathname = NULL; 
 
-	filechooser = gtk_file_chooser_dialog_new (_("Save a Copy"),
-						   NULL,
-						   GTK_FILE_CHOOSER_ACTION_SAVE,
-						   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-						   GTK_STOCK_SAVE, GTK_RESPONSE_OK,
-						   NULL);
-	gtk_window_set_modal (GTK_WINDOW (filechooser), TRUE);
+	priv = GPDF_CONTROL (control)->priv;
+	priv->file_chooser = gtk_file_chooser_dialog_new (
+		_("Save a Copy"),
+		NULL, GTK_FILE_CHOOSER_ACTION_SAVE,
+		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+		GTK_STOCK_SAVE, GTK_RESPONSE_OK,
+		NULL);
+	gtk_window_set_modal (GTK_WINDOW (priv->file_chooser), TRUE);
+
 	pdf_filter = gtk_file_filter_new ();
 	gtk_file_filter_set_name (pdf_filter, _("PDF Documents"));
 	gtk_file_filter_add_mime_type (pdf_filter, "application/pdf");
-	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (filechooser), pdf_filter);
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (priv->file_chooser),
+				     pdf_filter);
+
 	all_filter = gtk_file_filter_new ();
 	gtk_file_filter_set_name (all_filter, _("All Files"));
 	gtk_file_filter_add_pattern (all_filter, "*");
-	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (filechooser), all_filter);
-	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (filechooser), pdf_filter);
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (priv->file_chooser),
+				     all_filter);
+	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (priv->file_chooser),
+				     pdf_filter);
 	
-	gtk_dialog_set_default_response (GTK_DIALOG (filechooser), GTK_RESPONSE_OK);
+	gtk_dialog_set_default_response (GTK_DIALOG (priv->file_chooser),
+					 GTK_RESPONSE_OK);
 
 	bonobo_control_set_transient_for (control,
-					  GTK_WINDOW (filechooser),
+					  GTK_WINDOW (priv->file_chooser),
 					  NULL);
-	g_signal_connect_swapped (G_OBJECT (control), "disconnected",
-				  G_CALLBACK (gtk_widget_destroy),
-				  filechooser);
-	gtk_widget_show (filechooser);
+	gtk_widget_show (priv->file_chooser);
 
-	if (gtk_dialog_run(GTK_DIALOG (filechooser)) == GTK_RESPONSE_OK)
-		pathname = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (filechooser));
+	if (gtk_dialog_run(GTK_DIALOG (priv->file_chooser)) == GTK_RESPONSE_OK)
+		pathname = gtk_file_chooser_get_filename (
+			GTK_FILE_CHOOSER (priv->file_chooser));
 
-	gtk_widget_destroy (filechooser); 
+	gtk_widget_destroy (priv->file_chooser);
+	priv->file_chooser = NULL;
 	
 	return pathname; 
 }
@@ -1085,32 +1095,48 @@ verb_SaveAs_cb (BonoboUIComponent *uic, gpointer user_data,
 	g_free (filename);
 }
 
+static void
+properties_response_handler (GtkDialog *dialog, gint response, gpointer data)
+{
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+	GPDF_CONTROL (data)->priv->properties = NULL;
+}
+
+static gint
+properties_delete_handler (GtkWidget *widget, GdkEventAny *even,
+			   gpointer data)
+{
+	GPDF_CONTROL (data)->priv->properties = NULL;
+	return FALSE; /* destroy it in the default handler */
+}
+
 static void
 verb_FileProperties_cb (BonoboUIComponent *uic, gpointer user_data,
 			const char *cname)
 {
-	GtkWidget *dialog;
+	GPdfControlPrivate *priv;
 	GtkWidget *props;
 
 	g_return_if_fail (GPDF_IS_NON_NULL_CONTROL (user_data));
 
-	dialog = gtk_dialog_new_with_buttons (_("PDF Properties"),
-					      NULL,
-					      GTK_DIALOG_NO_SEPARATOR,
-					      GTK_STOCK_CLOSE,
-					      GTK_RESPONSE_CLOSE,
-					      NULL);
-	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
-	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+	priv = GPDF_CONTROL (user_data)->priv;
 
-	g_signal_connect (GTK_WIDGET (dialog), "response",
-			  G_CALLBACK (gtk_widget_destroy), NULL);
-	g_signal_connect_swapped (G_OBJECT (user_data), "disconnected",
-				  G_CALLBACK (gtk_widget_destroy),
-				  GTK_WIDGET (dialog));
+	priv->properties = gtk_dialog_new_with_buttons (_("PDF Properties"),
+							NULL,
+							GTK_DIALOG_NO_SEPARATOR,
+							GTK_STOCK_CLOSE,
+							GTK_RESPONSE_CLOSE,
+							NULL);
+	gtk_container_set_border_width (GTK_CONTAINER (priv->properties), 5);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (priv->properties)->vbox), 2);
+	gtk_window_set_resizable (GTK_WINDOW (priv->properties), FALSE);
+
+	g_signal_connect (priv->properties, "response",
+			  G_CALLBACK (properties_response_handler), user_data);
+	g_signal_connect (priv->properties, "delete-event",
+			  G_CALLBACK (properties_delete_handler), user_data);
 	bonobo_control_set_transient_for (BONOBO_CONTROL (user_data),
-					  GTK_WINDOW (dialog), NULL);
+					  GTK_WINDOW (priv->properties), NULL);
 
 	props = GTK_WIDGET (g_object_new (GPDF_TYPE_PROPERTIES_DISPLAY, NULL));
 	pdf_doc_process_properties (
@@ -1118,8 +1144,9 @@ verb_FileProperties_cb (BonoboUIComponent *uic, gpointer user_data,
 		G_OBJECT (props));
 
 	gtk_widget_show (props);
-	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), props);
-	gtk_widget_show (dialog);
+	gtk_box_pack_start_defaults (
+		GTK_BOX (GTK_DIALOG (priv->properties)->vbox), props);
+	gtk_widget_show (priv->properties);
 }
 
 static void
@@ -1906,19 +1933,8 @@ persist_file_loading_finished_cb (GPdfPersistFile *persist_file,
 	vfs_uri = uri ? gnome_vfs_uri_new (uri) : NULL;
 
 	if (vfs_uri) {
-		/* Fix for bug #123638: unescape title */
-		const char *title =
-			gnome_vfs_uri_to_string
-			    (vfs_uri,
-			     (GnomeVFSURIHideOptions)
-			     (GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD |
-			      GNOME_VFS_URI_HIDE_HOST_PORT |
-			      GNOME_VFS_URI_HIDE_HOST_NAME |
-			      GNOME_VFS_URI_HIDE_PASSWORD |
-			      GNOME_VFS_URI_HIDE_USER_NAME));
 		control->priv->title =
-			gnome_vfs_unescape_string_for_display (title);
-		g_free ((gpointer)title); 
+			gnome_vfs_uri_extract_short_name (vfs_uri);
 		gnome_vfs_uri_unref (vfs_uri);
 	} else {
 		control->priv->title = g_strdup (_("PDF Document"));
@@ -2450,6 +2466,22 @@ gpdf_control_disconnected (BonoboControl *control)
 	priv = GPDF_CONTROL (control)->priv;
 
 	gpdf_control_save_sidebar_state (GPDF_CONTROL (control));
+
+	if (priv->file_chooser) {
+		gtk_widget_destroy (priv->file_chooser);
+		priv->file_chooser = NULL;
+	}
+
+	if (priv->properties) {
+		gtk_widget_destroy (priv->properties);
+		priv->properties = NULL;
+	}
+
+	if (priv->print_dialog) {
+		gtk_widget_destroy (priv->print_dialog);
+		priv->print_dialog = NULL;
+	}
+
 	BONOBO_CALL_PARENT (BONOBO_CONTROL_CLASS, disconnected, (control));
 }
 
