@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; c-indent-level: 8 -*- */
 /*  PDF view widget
  *
  *  Copyright (C) 2002 Martin Kretzschmar
@@ -444,18 +444,18 @@ gpdf_view_link_clicked_cb (GPdfLinksCanvasLayer *links_layer,
 static gboolean
 gpdf_view_reset_link_status_cb (gpointer user_data)
 {
-    GPdfView *gpdf_view;
-    GPdfViewPrivate *priv;
-    
-    g_return_val_if_fail (GPDF_IS_VIEW (user_data), FALSE);
-
-    gpdf_view = GPDF_VIEW (user_data);
-    priv = gpdf_view->priv;
-
-    gpdf_control_private_clear_stack (priv->parent);
-    priv->status_to_id = 0;
-    
-    return FALSE;
+	GPdfView *gpdf_view;
+	GPdfViewPrivate *priv;
+	
+	g_return_val_if_fail (GPDF_IS_VIEW (user_data), FALSE);
+	
+	gpdf_view = GPDF_VIEW (user_data);
+	priv = gpdf_view->priv;
+	
+	gpdf_control_private_clear_stack (priv->parent);
+	priv->status_to_id = 0;
+	
+	return FALSE;
 }
 
 static void
@@ -622,7 +622,7 @@ gpdf_view_link_leaved_cb (GPdfLinksCanvasLayer *links_layer,
 	}
 }
 
-static gboolean
+static void
 gpdf_view_render_page (GPdfView *gpdf_view)
 {
 	GPdfViewPrivate *priv;
@@ -637,12 +637,12 @@ gpdf_view_render_page (GPdfView *gpdf_view)
 	gdouble page_transform [6];
 	Object obj;
 
-	g_return_val_if_fail (GPDF_IS_VIEW_WITH_DOC (gpdf_view), FALSE);
+	g_return_if_fail (GPDF_IS_VIEW_WITH_DOC (gpdf_view));
 
 	priv = gpdf_view->priv;
 
 	if (priv->requested_page == priv->current_page)
-		return FALSE;
+		return;
 
 	gpdf_control_private_set_wait_cursor (priv->parent);
 	
@@ -740,7 +740,7 @@ gpdf_view_render_page (GPdfView *gpdf_view)
 			  gpdf_view);
 
 	gpdf_control_private_reset_cursor (priv->parent);
-	return FALSE;
+	return;
 }
 
 static void
@@ -762,7 +762,7 @@ gpdf_view_goto_page_no_history (GPdfView *gpdf_view, gint page)
 
 	priv->requested_page = CLAMP (page, 1, last_page);
 	gpdf_view_emit_page_changed (gpdf_view, priv->requested_page);
-	g_idle_add ((GSourceFunc)gpdf_view_render_page, gpdf_view);
+	gpdf_view_render_page (gpdf_view);
 }
 
 void
@@ -777,25 +777,43 @@ gpdf_view_goto_page (GPdfView *gpdf_view, gint page)
 void
 gpdf_view_scroll_to_top (GPdfView *gpdf_view)
 {
-	gint x, y;
+	GnomeCanvas *canvas;
+	gint x;
 
 	g_return_if_fail (GPDF_IS_NON_NULL_VIEW (gpdf_view));
 
-	gnome_canvas_get_scroll_offsets (GNOME_CANVAS (gpdf_view), &x, &y);
-	gnome_canvas_scroll_to (GNOME_CANVAS (gpdf_view), x, 0);
+	canvas = GNOME_CANVAS (gpdf_view);
+	gnome_canvas_get_scroll_offsets (canvas, &x, NULL);
+
+	gdk_window_set_back_pixmap (GTK_WIDGET (canvas)->window, NULL, FALSE);
+	gdk_window_hide (canvas->layout.bin_window);
+
+	gnome_canvas_scroll_to (canvas, x, 0);
+
+	gdk_window_show (canvas->layout.bin_window);
+	gtk_style_set_background (GTK_WIDGET (canvas)->style, GTK_WIDGET (canvas)->window, GTK_STATE_NORMAL);
 }
 
 void
 gpdf_view_scroll_to_bottom (GPdfView *gpdf_view)
 {
+	GnomeCanvas *canvas;
 	gint x, y;
 
 	g_return_if_fail (GPDF_IS_NON_NULL_VIEW (gpdf_view));
 
-	gnome_canvas_get_scroll_offsets (GNOME_CANVAS (gpdf_view), &x, &y);
-	gnome_canvas_scroll_to (GNOME_CANVAS (gpdf_view), x,
-				(GTK_LAYOUT (gpdf_view)->height
-				 - GTK_WIDGET (gpdf_view)->allocation.height));
+	canvas = GNOME_CANVAS (gpdf_view);
+	gnome_canvas_get_scroll_offsets (canvas, &x, NULL);
+
+	y = canvas->layout.height - GTK_WIDGET (gpdf_view)->allocation.height;
+
+	gdk_window_set_back_pixmap (GTK_WIDGET (canvas)->window, NULL, FALSE);
+	gdk_window_hide (canvas->layout.bin_window);
+
+	gnome_canvas_scroll_to (canvas, x, y);
+
+	gdk_window_show (canvas->layout.bin_window);
+	gtk_style_set_background (GTK_WIDGET (canvas)->style, GTK_WIDGET (canvas)->window, GTK_STATE_NORMAL);
 }
 
 gint
@@ -1024,8 +1042,19 @@ gpdf_view_zoom (GPdfView *gpdf_view, gdouble factor, gboolean relative)
 	}
 
 	priv->zoom = CLAMP (zoom, MIN_ZOOM, MAX_ZOOM);
+
+	gdk_window_set_back_pixmap (GTK_WIDGET (gpdf_view)->window, NULL,
+				    FALSE);
+ 	gdk_window_hide (GNOME_CANVAS (gpdf_view)->layout.bin_window);
+
 	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (gpdf_view),
 					  priv->zoom);
+
+ 	gdk_window_show (GNOME_CANVAS (gpdf_view)->layout.bin_window);
+	gtk_style_set_background (GTK_WIDGET (gpdf_view)->style,
+				  GTK_WIDGET (gpdf_view)->window,
+				  GTK_STATE_NORMAL);
+
 	gpdf_view_emit_zoom_changed (gpdf_view, priv->zoom);
 }
 
@@ -1346,12 +1375,17 @@ gpdf_view_handle_drag_motion (GPdfView *gpdf_view, GdkEventMotion *event)
 
 	priv = gpdf_view->priv;
 
+	gnome_canvas_get_scroll_offsets (GNOME_CANVAS (gpdf_view),
+			&priv->drag_offset_x, &priv->drag_offset_y);
+
 	if (event->is_hint) {
 		gdk_window_get_pointer (GTK_WIDGET (gpdf_view)->window,
 					&x, &y, &mod);
 	} else {
-		x = (gint)event->x;
-		y = (gint)event->y;
+		/* subtract drag_offset because event->x is relative to backing window
+		 * of the canvas and not the clipping window of the widget */
+		x = (gint)event->x - priv->drag_offset_x;
+		y = (gint)event->y - priv->drag_offset_y;
 	}
 
 	dx = priv->drag_anchor_x - x;
@@ -1361,11 +1395,10 @@ gpdf_view_handle_drag_motion (GPdfView *gpdf_view, GdkEventMotion *event)
 				priv->drag_offset_x + dx,
 				priv->drag_offset_y + dy);
 
-	priv->drag_anchor_x = (gint)event->x;
-	priv->drag_anchor_y = (gint)event->y;
-	gnome_canvas_get_scroll_offsets (GNOME_CANVAS (gpdf_view),
-					 &priv->drag_offset_x,
-					 &priv->drag_offset_y);
+	/* reset the anchor to the new position of the mouse */
+	priv->drag_anchor_x = x;
+	priv->drag_anchor_y = y;
+
 }
 
 static gboolean
